/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
use demyo;

DROP TABLE IF EXISTS demyo_authors;
DROP TABLE IF EXISTS demyo_series;
DROP TABLE IF EXISTS demyo_series_relations;
DROP TABLE IF EXISTS demyo_bindings;
DROP TABLE IF EXISTS demyo_albums;
DROP TABLE IF EXISTS demyo_publishers;
DROP TABLE IF EXISTS demyo_collections;
DROP TABLE IF EXISTS demyo_albums_artists;
DROP TABLE IF EXISTS demyo_albums_writers;
DROP TABLE IF EXISTS demyo_albums_colorists;
DROP TABLE IF EXISTS demyo_tags;
DROP TABLE IF EXISTS demyo_albums_tags;
DROP TABLE IF EXISTS demyo_images;
DROP TABLE IF EXISTS demyo_albums_images;
DROP TABLE IF EXISTS demyo_sources;
DROP TABLE IF EXISTS demyo_derivatives;
DROP TABLE IF EXISTS demyo_derivative_types;
DROP TABLE IF EXISTS demyo_derivatives_images;
DROP TABLE IF EXISTS demyo_borrowers;
DROP TABLE IF EXISTS demyo_albums_borrowers;

CREATE TABLE demyo_authors (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(127) NOT NULL,
	fname VARCHAR(127) NULL,
	nickname VARCHAR(127) NULL,
	biography TEXT NULL,
	website VARCHAR(255) NULL,
	INDEX(name)
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_series (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL,
	summary TEXT NULL,
	completed BOOLEAN DEFAULT false,
	comment TEXT NULL,
	website VARCHAR(255) NULL,
	INDEX(name)
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_series_relations (
	main INT UNSIGNED REFERENCES demyo_series(id),
	sub INT UNSIGNED REFERENCES demyo_series(id),
	PRIMARY KEY(main, sub),
	INDEX(main)
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_bindings (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_images (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	url VARCHAR(255) NULL,
	is_local BOOLEAN DEFAULT true,
	description TEXT NULL,
	INDEX(description(20))
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_albums (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	series_id INT UNSIGNED NOT NULL REFERENCES demyo_series(id),
	cycle SMALLINT UNSIGNED NULL,
	number FLOAT UNSIGNED NULL, /* Allowing things like '1a' involves unituitive sorting for users (11 is lower than 2) */
	title VARCHAR(255) NULL, /* One shots sometimes have no title */
	publisher_id INT UNSIGNED NOT NULL REFERENCES demyo_publishers(id),
	collection_id INT UNSIGNED NULL REFERENCES demyo_collections(id),
	first_edition DATE NULL,
	this_edition DATE NULL,
	isbn VARCHAR(18) NULL, /* ISBN-10 or ISBN-13 with at most 4 separators (spaces or dashes) */
	purchase_date DATE NULL,
	binding_id INT UNSIGNED NULL REFERENCES demyo_bindings(id),
	cover_id INT UNSIGNED NULL REFERENCES demyo_images(id),
	summary TEXT NULL,
	comment TEXT NULL,
	price FLOAT UNSIGNED NULL,
	height FLOAT UNSIGNED NULL,
	width FLOAT UNSIGNED NULL,
	pages INT UNSIGNED NULL,
	INDEX(title)
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_albums_images (
	album_id INT UNSIGNED NOT NULL REFERENCES demyo_albums(id),
	image_id INT UNSIGNED NOT NULL REFERENCES demyo_images(id),
	PRIMARY KEY(album_id, image_id)
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_publishers (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL,
	website VARCHAR(255) NULL,
	feed VARCHAR(255) NULL,
	history TEXT NULL,
	INDEX(name)
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_collections (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL,
	website VARCHAR(255) NULL,
	feed VARCHAR(255) NULL,
	history TEXT NULL,
	publisher_id INT UNSIGNED NOT NULL REFERENCES demyo_publishers(id),
	INDEX(name)
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_albums_artists (
	album_id INT UNSIGNED NOT NULL REFERENCES demyo_albums(id),
	artist_id INT UNSIGNED NOT NULL REFERENCES demyo_authors(id),
	PRIMARY KEY(album_id, artist_id)
) engine=InnoDB CHARACTER SET utf8;
CREATE TABLE demyo_albums_writers (
	album_id INT UNSIGNED NOT NULL REFERENCES demyo_albums(id),
	writer_id INT UNSIGNED NOT NULL REFERENCES demyo_authors(id),
	PRIMARY KEY(album_id, writer_id)
) engine=InnoDB CHARACTER SET utf8;
CREATE TABLE demyo_albums_colorists (
	album_id INT UNSIGNED NOT NULL REFERENCES demyo_albums(id),
	colorist_id INT UNSIGNED NOT NULL REFERENCES demyo_authors(id),
	PRIMARY KEY(album_id, colorist_id)
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_tags (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(127) NOT NULL UNIQUE
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_albums_tags (
	album_id INT UNSIGNED NOT NULL REFERENCES demyo_albums(id),
	tag_id INT UNSIGNED NOT NULL REFERENCES demyo_tags(id),
	PRIMARY KEY(album_id, tag_id)
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_sources (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(127) NOT NULL,
	owner VARCHAR(127) NULL,
	email VARCHAR(255) NULL,
	website VARCHAR(255) NULL,
	address VARCHAR(255) NULL,
	phone_number VARCHAR(31) NULL,
	history TEXT NULL
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_derivatives (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	series_id INT UNSIGNED NOT NULL REFERENCES demyo_series(id),
	album_id INT UNSIGNED NULL REFERENCES demyo_albums(id),
	artist_id INT UNSIGNED NULL REFERENCES demyo_authors(id),
	derivative_type_id INT UNSIGNED NOT NULL REFERENCES demyo_derivative_types(id),
	colors SMALLINT UNSIGNED NULL,
	/* Source could be an editor, a library, the author himself. */
	source_id INT UNSIGNED NULL REFERENCES demyo_sources(id),
	number SMALLINT UNSIGNED NULL,
	total SMALLINT UNSIGNED NULL, 
	signed BOOLEAN DEFAULT true,
	authors_copy BOOLEAN DEFAULT false,
	restricted_sale BOOLEAN DEFAULT false,
	description TEXT NULL,
	width float UNSIGNED NULL,
	height float UNSIGNED NULL,
	depth float UNSIGNED NULL,
	purchase_date DATE NULL,
	price FLOAT UNSIGNED NULL
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_derivative_types (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(127) NOT NULL
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_derivatives_images (
	derivative_id INT UNSIGNED NOT NULL REFERENCES demyo_derivatives(id),
	image_id INT UNSIGNED NOT NULL REFERENCES demyo_images(id),
	PRIMARY KEY(derivative_id, image_id)
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_borrowers (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(127) NOT NULL,
	fname VARCHAR(127) NULL,
	email VARCHAR(255) NULL,
	UNIQUE(name, fname)
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE demyo_albums_borrowers (
	album_id INT UNSIGNED NOT NULL REFERENCES demyo_albums(id),
	borrower_id INT UNSIGNED NOT NULL REFERENCES demyo_borrowers(id),
	borrow_date DATE NOT NULL,
	return_date DATE NULL,
	PRIMARY KEY(album_id, borrower_id, borrow_date)
) engine=InnoDB CHARACTER SET utf8;

SELECT 'done';
