<?php
/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class CollectionsController extends AppController {
	public $scaffold;
	public $uses = array('Collection', 'Album', 'Publisher');
	public $helpers = array('Menu', 'Quicksearch');
	
	public $paginate = array('limit' => 20, 'page' => 1,
		'fields' => array('Collection.id', 'Collection.name', 'Publisher.id', 'Publisher.name'),
		'order'=>array('Publisher.name' => 'ASC', 'Collection.name' => 'ASC'), 'recursive' => 1);
	
	public function index() {
		$this->Collection->unbindAll(array('belongsTo' => array('Publisher')), false);
		
		// Fix #57 - Collection index doesn't show Publishers with no Collection
		$this->Collection->belongsTo['Publisher']['type'] = 'RIGHT OUTER';
		
		$collections =  $this->paginate('Collection');
		$this->set('collections', $collections);
	}

	public function view($id = null) {
		if ($id === null)
			$this->redirect('/collections');

		$this->Collection->id = $id;
		$this->Collection->recursive = 1;
		$this->Collection->unbindAll(array('belongsTo' => array('Publisher')), false);
		$collection = $this->Collection->read();
		$collection['Collection']['album_count'] = $this->Album->getAlbumCount('Collection', $id);
		
		$this->set('collection', $collection);
	}
	
	public function add() {
		if (!empty($this->data)) {
			if ($this->Collection->save($this->data)) {
				$this->smartFlash(__('The collection has been added', true),
					'/collections/view/'.$this->Collection->id);
			}
		}
		
		else
			$this->generateLists();
	}
	
	public function edit($id = null) {
		if (empty($this->data)) {
			$this->Collection->id = $id;
			$this->data = $this->Collection->read();
			$this->generateLists();
		}
		
		else {
			if ($this->Collection->save($this->data)) {
				$this->smartFlash(__('The collection has been updated', true),
					'/collections/view/'.$this->data['Collection']['id']);
			}
		}
	}
	
	private function generateLists() {
		$this->set('publishersList', $this->Publisher->find('list'));
	}
	
	function fromPublisher($id = null) {
		$this->layout = 'ajax';
		
		if ($id != null)
			$this->set('collections', $this->Collection->find('list', array('conditions' => 'Collection.publisher_id='.$id)));
		else
			$this->set('collections', array());
	}
}

?>
