<?php
/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class CSVImporterComponent extends Object {
	private $warnings;
	private $controller;
	
	/** Imports data from a CSV file into the database.
	 * Expected format :
	 * series;cycle;number;title;artist;writer;colorist;publisher;collection;first_edition;this_edition;purchase_date;binding;summary;comment;price
	 */
	// TODO: we can probably cache some results
	public function import(&$controller, $filename) {
		$csv = fopen($filename, 'r');
		$total = 0;
		
		$this->warnings = array();
		$this->controller =& $controller;
		
		// Delete all
		$controller->Publisher->deleteAll('1 = 1');
		$controller->Collection->deleteAll('1 = 1');
		$controller->Binding->deleteAll('1 = 1');
		$controller->Series->deleteAll('1 = 1');
		$controller->Author->deleteAll('1 = 1');
		$controller->Album->deleteAll('1 = 1');

		// We'll build this step by step
		$adata = null;

		while(($line = fgetcsv($csv, 0, ';', '"')) !== false) {
			$total++;
			unset($adata);
			// Load fields
			list($series, $cycle, $number, $title, $artist, $writer,
				$colorist, $publisher, $collection, $first_edition,
				$this_edition, $purchase_date, $binding, $summary, $comment,
				$price) = $line;

			// Fix some of them
			$series = ucwords(strtolower($series));
			$title = ucwords(strtolower($title));
			$artist = ucwords(strtolower($artist));
			$writer = ucwords(strtolower($writer));
			$colorist = ucwords(strtolower($colorist));
			$publisher = ucwords(strtolower($publisher));
			$collection = ucwords(strtolower($collection));

			// Set trivial information
			foreach (array('cycle', 'title', 'number', 'summary', 'comment') as $trivial)
				if (strlen(${$trivial}) > 0)
					$adata['Album'][$trivial] = ${$trivial};
			if (strlen($price)>0 && $price > 0)
				$adata['Album']['price'] = $price;

			// Check names
			list($artist_name, $artist_fname) = $this->parseName($artist);
			list($writer_name, $writer_fname) = $this->parseName($writer);
			list($colorist_name, $colorist_fname) = $this->parseName($colorist);
			
			// Check dates
			if (strlen($first_edition = $this->fixDate($first_edition))>0)
				$adata['Album']['first_edition'] = $first_edition;
			if (strlen($this_edition = $this->fixDate($this_edition))>0)
				$adata['Album']['this_edition'] = $this_edition;
			if (strlen($purchase_date = $this->fixDate($purchase_date))>0)
				$adata['Album']['purchase_date'] = $purchase_date;

			// Search for items or insert them {{{
			// Search for publisher, or insert it
			if (strlen($publisher) > 0) {
				$publisher_id = $this->getIdOrInsert('Publisher',
					array('name'=>$publisher));
				$adata['Album']['publisher_id'] = $publisher_id;
			} else {
				$this->warning($number, $line, "Empty publisher");
				continue;
			}

			// Search for collection, or insert it
			if (strlen($collection) > 0) {
				$collection_id = $this->getIdOrInsert('Collection',
					array('name'=>$collection, 'publisher_id'=>$publisher_id));
				$adata['Album']['collection_id'] = $collection_id;
			} // Drop empty
			
			// Search for binding, or insert it
			if (strlen($binding) > 0) {
				$binding_id = $this->getIdOrInsert('Binding',
					array('name'=>$binding));
				$adata['Album']['binding_id'] = $binding_id;
			} // Drop empty

			// Search for series, or insert it
			if (strlen($series) > 0) {
				$series_id = $this->getIdOrInsert('Series',
					array('name'=>$series));
				$adata['Album']['series_id'] = $series_id;
			} else {
				$this->warning($number, $line, "Empty series");
				continue;
			}
			
			// Search for artist, or insert it
			if ($artist_name != '') {
					$artist_id = $this->getIdOrInsert('Artist',
						array('name'=>$artist_name, 'fname'=>$artist_fname));
				$adata['Artist']['Artist'] = array($artist_id);
			} // Drop empty
			
			// Search for writer, or insert it
			if ($writer_name != '') {
					$writer_id = $this->getIdOrInsert('Writer',
						array('name'=>$writer_name, 'fname'=>$writer_fname));
				$adata['Writer']['Writer'] = array($writer_id);
			} // Drop empty
			
			// Search for colorist, or insert it
			if ($colorist_name != '') {
					$colorist_id = $this->getIdOrInsert('Colorist',
						array('name'=>$colorist_name, 'fname'=>$colorist_fname));
				$adata['Colorist']['Colorist'] = array($colorist_id);
			} // Drop empty
			// }}}

			$controller->Album->create();
			if ($controller->Album->save($adata) === false)
				$this->warning($total, $line, 'Failed insertion for unknown reason');
		}
		fclose($csv);

		return $this->warnings;
	}
	
	private function parseName($name) {
		$name = trim($name);
		
		$words = explode(' ',$name); // Guaranteed to return non-false
		if (sizeof($words) == 1) {
			$lastName = $words[0];
			$firstName = '';
		} else if (sizeof($words) == 2) {
			$lastName = $words[0];
			$firstName = $words[1];
		} else {
			$lastName = $name;
			$firstName = '';
		}
		
		return array($lastName, $firstName);
	}

	/** Returns a fixed date, or an empty string if an error occurred
	 * Expects YYYY-MM-DD, YYYY-MM or YYYY
	 */
	private function fixDate($date) {
		if (preg_match('/^\\d{4}-\\d{2}-\\d{2}$/', $date))
			return $date;
		else if (preg_match('/^\\d{4}-\\d{2}$/', $date))
			return "$date-01";
		else if (preg_match('/^\\d{4}$/', $date))
			return "$date-01-01";
		else if (preg_match('+^(\\d{2})/(\\d{2})/(\\d{4})$+', $date, $matches))
			return "{$matches[3]}-{$matches[2]}-{$matches[1]}";
		else
			return '';
	}

	private function getIdOrInsert($modelName, $fields) {
		$this->controller->$modelName->recursive = -1;
		$entry = $this->controller->$modelName->find('first', array('conditions' => $fields, 'fields' => array('id')));
		if ($entry === false) { // Insert
			foreach($fields as $key=>$value)
				$data[$modelName][$key] = $value;
			$this->controller->$modelName->create();
			$this->controller->$modelName->save($data, false);
			return $this->controller->$modelName->getLastInsertID();
		} else
			return $entry[$modelName]['id'];
	}

	private function warning($number, $line, $message, $echo = false) {
		$line = implode(';',$line);
		$str = "Warning: line $number dropped ~ $message (line was $line)";
		if ($echo)
			debug($str);
		$this->warnings[] = $str;
	}
}
?>