<?php
/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class SeriesController extends AppController {
	public $scaffold;
	public $helpers = array('jQuery', 'List', 'Menu', 'Quicksearch', 'Shadowbox');
	public $uses = array('Series', 'Album', 'Derivative');
	public $components = array('PhpThumb');
	public $paginate = array('limit' => 45, 'page' => 1, 'order'=>array('name' => 'asc'), 'fields' => array('id','name','completed'), 'recursive' => 0); 

	function index() {
		$this->set('series', $this->paginate('Series'));
		$this->set('perColumn', ceil($this->paginate['limit'] / 3));
	}

	function view($id = null) {
		if ($id == null)
			$this->redirect('/series');

		$this->Series->id = $id;
		$this->Series->recursive = 2;
		
		// Fetch only vital information
		$this->Series->contain(array(
			'Album', 'Album.Publisher', 'Album.Collection', 'Album.Binding',
				'Album.Artist', 'Album.Writer', 'Album.Colorist', 'Album.Tag',
				'Album.Cover', 'Album.Borrower',
			'RelatedSeries'
		));
		
		$series = $this->Series->read();
		$this->pageTitle = __s('Viewing series: %1$s', $series['Series']['name']);
		
		// Look for related editors, make thumbnails
		$editors = array();
		foreach($series['Album'] as &$album) {
			
			if (! isset($editors[$album['Publisher']['id']]))
				$editors[$album['Publisher']['id']] = $album['Publisher'];
			
			if (! empty($album['Cover']['id']))
				$album['Cover']['thumb_url'] = $this->PhpThumb->getThumbnail(IMAGES.$album['Cover']['url']);
		}
		$this->set('editors', $editors);
		
		$series['Series']['derivative_count'] = $this->Derivative->getDerivativesCount('Series', $id);
		$this->set('series', $series);
		
		// Look for related authors
		$this->set('artists', $this->getAuthors($series['Album'], 'Artist'));
		$this->set('writers', $this->getAuthors($series['Album'], 'Writer'));
		$this->set('colorists', $this->getAuthors($series['Album'], 'Colorist'));
    }
	
	private function getAuthors($album_list, $author_type) {
		$authors = array();
		foreach($album_list as $album)
			foreach ($album[$author_type] as $author)
				if (! isset($authors[$author['id']]))
					$authors[$author['id']] = $author;
		return $authors;
	}
	
	function add() {
		if (!empty($this->data)) {
			if ($this->Series->save($this->data)) {
				$this->smartFlash(__('The series has been added', true),
					'/series/view/'.$this->Series->id);
			}
		}
		
		else
			$this->generateLists();
	}
	
	function edit($id = null) {
		if (empty($this->data)) {
			$this->Series->id = $id;
			$this->data = $this->Series->read();
			$this->generateLists();
		}
		
		else {
			if ($this->Series->save($this->data)) {
				$this->smartFlash(__('The series has been updated', true),
					'/series/view/'.$this->data['Series']['id']);
			}
		}
	}
	
	function generateLists() {
		$this->set('seriesList', $this->Series->find('list'));
	}
}

?>
