<?php
/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class Author extends AppModel {
	public $actsAs = array('Containable', 'SimpleXML');
	public $name = 'Author';
	public $order = 'name ASC, fname ASC';
	
	public $validate = array(
		'name'  => VALID_NOT_EMPTY
	);
	
	function find($conditions = null, $fields = array(), $order = null, $recursive = null) {
		if ($conditions === 'list') {
			$fields = array_merge(array('conditions' => null, 'limit' => null), $fields);
			$results = $this->findAll($fields['conditions'], 'Author.id, Author.name, Author.nickname, Author.fname', $this->order, $fields['limit'], 1, -1);

			$list = array();
			foreach ($results as $item) {
				$list[$item['Author']['id']] = $item['Author']['name']
					.(empty($item['Author']['fname'])?'':(', '.$item['Author']['fname']))
					.(empty($item['Author']['nickname'])?'':(' ('.$item['Author']['nickname'].')'));
			}

			return $list;
		}
		
		// Authors are subclassed so we must pay attention to the three roles
		else if ($conditions === 'orphans') {
			if ($this->name !== 'Author')
				return parent::find('orphans', $fields);
				
			$artist = new Artist();
			$aid = Set::extract('/Artist/id', $artist->find('orphans', $fields));
			$writer = new Writer();
			$wid = Set::extract('/Writer/id', $writer->find('orphans', $fields));
			$colorist = new Colorist();
			$cid = Set::extract('/Colorist/id', $colorist->find('orphans', $fields));
			
			
			$ids = array_intersect(array_intersect($aid, $wid), $cid);
			$this->unbindAll();
			return $this->find('all', array('conditions' => array("Author.id" => $ids)));
		}

		return parent::find($conditions, $fields, $order, $recursive);
	}
	
	/** After find callback, builds a meta_name based on the Author's name,
	 * first name and nick.
	 *
	 * @param mixed $results The results of the find operation
	 * @param boolean $primary Whether this model is being queried directly
	 * (vs. being queried as an association)
	 * @return mixed Result of the find operation
	 * @access public
	 */
	public function afterFind($results, $primary = false) {
		foreach ($results as &$result) {
			if (! is_array($result))
				continue;
			
			if (isset($result[$this->name])) // result was ['Model' => ...]
				$data =& $result[$this->name];
			else if (isset($result['name'])) // result was ['field1', 'field2']
				$data =& $result;
			else
				continue;
			
			if (empty($data)) // no matching record
				continue;
			
			if (! is_array(current($data)) && isset($data['name'])) { // result was ['Model' => ['field1', 'field2']]  
				$data['meta_name'] = trim(
					(empty($data['fname'])?'':"{$data['fname']} ")
					.(empty($data['nickname'])?'':('\''.$data['nickname'].'\' '))
					.$data['name']);
			}
			// else result was ['Model' => [0 => ['field1', 'field2']]], already
			// handled in a previous method call (the call for the sub-items)
		}
		
		return $results;
	}
	
	/** Returns the meta_name of the selected Author
	 *
	 * @param integer $id The ID of the Author
	 * @return string
	 */
	public function getName($id = null) {
		if ($id === null)
			$id = $this->id;
		
		$data = $this->find('first', array(
			'fields' => array('name', 'fname', 'nickname'),
			'conditions' => array($this->name.'.id' => $id),
			'recursive' => -1
		));
		
		return $data[$this->name]['meta_name'];
	}
}

?>
