<?php
/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class Borrower extends AppModel {
	public $name = 'Borrower';
	public $actsAs = array('SimpleXML');
	public $hasAndBelongsToMany = array(
		'Album' => array(
			'className' => 'Album',
			'condition' => '(AlbumsBorrower.return_date IS NULL OR AlbumsBorrower.return_date=\'0000-00-00\')',
			'with' => 'AlbumsBorrower'
		)
	);
	
	/** Gets the ID of a borrower, inserting him if he didn't already exist.
	 *
	 * @param string $name A string of the following forms : "NAME"
	 * or "NAME FIRST_NAME" or "NAME &lt;EMAIL&gt;"
	 * or "NAME FIRST_NAME &lt;EMAIL&gt;"
	 * @return integer The borrower ID
	 */
	public function getIDOrAdd($name) {
		preg_match('/^([^<> ]+)( ([^<> ]+))?( <(.+)>)?$/', trim($name), $matches);
		
		$numMatches = sizeof($matches);
		if ($numMatches == 2)
			$data['name'] = $matches[1];
		else if ($numMatches == 4) {
			$data['fname'] = $matches[1];
			$data['name'] = $matches[3];
		} else if ($numMatches == 6 && ! empty($matches[3])) {
			$data['fname'] = $matches[1];
			$data['name'] = $matches[3];
			$data['email'] = $matches[5];
		} else if ($numMatches == 6) {
			$data['name'] = $matches[1];
			$data['email'] = $matches[5];
		} else
			$data['name'] = $matches[0];
			
		// search for data before saving the record, maybe it isn't a new one
		$conditions = array('name' => $data['name']);
		if (isset($data['fname']))
			$conditions['fname'] = $data['fname'];
		// Don't check email
		
		$borrower = $this->find($conditions, 'id', null, -1);
		
		if (!empty($borrower))
			return (int)$borrower['Borrower']['id'];
		
		$this->save($data);
		return (int)($this->getInsertID());
	}
	
	/** After find callback, builds a meta_name based on the Borrower's name,
	 * and first name.
	 *
	 * @param mixed $results The results of the find operation
	 * @param boolean $primary Whether this model is being queried directly
	 * (vs. being queried as an association)
	 * @return mixed Result of the find operation
	 * @access public
	 */
	public function afterFind($results, $primary = false) {
		if (isset($results['name'])) { // $results was ['field1', 'field2']
			$this->buildName($results);
			return $results;
		}
		
		foreach ($results as &$result) {
			if (! is_array($result))
				continue;
			
			if (is_array($result) && isset($result[$this->name])) // result was ['Model' => ...]
				$data =& $result[$this->name];
			else if (isset($result['name'])) // result was ['field1', 'field2']
				$data =& $result;
			else
				continue;
			
			if (empty($data)) // no matching record or not enough data
				continue;
			
			if (! is_array(current($data))) { // result was ['Model' => ['field1', 'field2']]  
				$this->buildName($data);
			}
			// else result was ['Model' => [0 => ['field1', 'field2']]], already
			// handled in a previous method call (the call for the sub-items)
		}
		
		return $results;
	}
	
	private function buildName(&$data) {
		if (! isset($data['name']))
			return;
		
		$data['meta_name'] =
			empty($data['fname'])
			?($data['name'])
			:($data['fname'].' '.$data['name']);
	}
}

?>
