<?php
/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class Derivative extends AppModel {
	public $name = 'Derivative';
	public $actsAs = array('SimpleXML' => array('useHABTM' => true));
	public $belongsTo = array('Series', 'Album', 'DerivativeType', 'Artist', 'Source');
	public $hasAndBelongsToMany = array(
		'Image' => array('className' => 'Image')
	);
	
	public function getWorth($conditions = null) {
		$sum = $this->find($conditions, "SUM(price)", null, -1);
		return (float)$sum[0]['SUM(price)'];
	}
	
	/* After find callback, builds a meta_name based on the type, source and
	 * Series.
	 *
	 * @param mixed $results The results of the find operation
	 * @param boolean $primary Whether this model is being queried directly (vs. being queried as an association)
	 * @return mixed Result of the find operation
	 * @access public
	 */
	public function afterFind($results, $primary = false) {
		foreach ($results as &$result) {
			if (isset($result[$this->name])) { // result was ['Model' => ...]
				$data =& $result[$this->name];
			}
			else if (isset($result['source_id'])) // result was ['field1', 'field2']
				$data =& $result;
			else
				continue;
			
			$series = '';
			if (isset($data['Series']) && isset($data['Series']['name']))
				$series = $data['Series']['name'];
			else if (isset($result['Series']) && isset($result['Series']['name']))
				$series = $result['Series']['name'];
			
			$type = '';
			if (isset($data['DerivativeType']) && isset($data['DerivativeType']['name']))
				$type = $data['DerivativeType']['name'];
			else if (isset($result['DerivativeType']) && isset($result['DerivativeType']['name']))
				$type = $result['DerivativeType']['name'];
			
			$source = '';
			if (isset($data['Source']) && isset($data['Source']['name']))
				$source = $data['Source']['name'];
			else if (isset($result['Source']) && isset($result['Source']['name']))
				$source = $result['Source']['name'];

			if (empty($data)) // no matching record
				continue;
			
			if (! is_array(current($data))) { // result was ['Model' => ['field1', 'field2']]
				if ($series != '' && $type != '') { // $type is mandatory, but could be not part of the selected fields for this query
					if ($source != '')
						$data['meta_name'] = __s('%1$s for %2$s from %3$s', $type, $series, $source);
					else
						$data['meta_name'] = __s('%1$s for %2$s', $type, $series);
				}
			}
			// else result was ['Model' => [0 => ['field1', 'field2']]], already
			// handled in a previous method call (the call for the sub-items)
		}
		
		return $results;
	}
	
	public function getDerivativesCount($model, $id) {
		if (! isset($this->belongsTo[$model])) {
			$this->log("$model is not a associated to {$this->name}", LOG_WARNING);
			return 0;
		}
		
		$model = Inflector::underscore($model);
		return $this->find('count', array(
			'conditions' => array("Derivative.{$model}_id" => $id),
			'recursive' => -1
		));
	}
}

?>
