<?php
/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */

/** Helper for common operations on plain text
 */
class TextHelper extends AppHelper {
	/** Shortens a text to a given length, trying to keep full words.
	 * The returned string stops at the last word that macthes the length
	 * restriction.
	 * Examples:
	 *  shortenWithFullWords('John Doe', 8); // John Doe
	 *  shortenWithFullWords('John Doe', 7); // John...
	 *  shortenWithFullWords('JohnDoe', 6); // JohnDo...
	 *
	 * @param string $input The string to shorten
	 * @param int $characterCount The maxiumum length of the string
	 * (not counting $append)
	 * @param string $append A text to append if the string is shortened
	 * @return string The shortened string
	 */
	public function shortenWithFullWords($input, $characterCount, $append = '...') {
		if ($characterCount >= strlen($input) || $characterCount === null
				|| $input === null)
			return $input;
		
		$offset = $characterCount;
		for (; $offset >= 0 && $input{$offset} !== ' '; $offset--);
		
		if ($offset <= 0) 
			$offset = $characterCount; // shorten abruptly
		
		return substr($input, 0, $offset).$append;
	}
}

?>